# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/summary.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_summary(object):
    def setupUi(self, summary):
        summary.setObjectName("summary")
        summary.resize(632, 254)
        self.summary_layout = QtWidgets.QHBoxLayout(summary)
        self.summary_layout.setObjectName("summary_layout")
        self.summary_options_layout = QtWidgets.QVBoxLayout()
        self.summary_options_layout.setObjectName("summary_options_layout")
        self.activity_types_list = CheckList(summary)
        self.activity_types_list.setMaximumSize(QtCore.QSize(300, 16777215))
        self.activity_types_list.setObjectName("activity_types_list")
        self.summary_options_layout.addWidget(self.activity_types_list)
        self.summary_period_layout = QtWidgets.QHBoxLayout()
        self.summary_period_layout.setObjectName("summary_period_layout")
        self.label = QtWidgets.QLabel(summary)
        self.label.setObjectName("label")
        self.summary_period_layout.addWidget(self.label)
        self.summary_period_box = QtWidgets.QComboBox(summary)
        self.summary_period_box.setMaximumSize(QtCore.QSize(300, 16777215))
        self.summary_period_box.setObjectName("summary_period_box")
        self.summary_period_box.addItem("")
        self.summary_period_box.addItem("")
        self.summary_period_box.addItem("")
        self.summary_period_box.addItem("")
        self.summary_period_layout.addWidget(self.summary_period_box)
        self.summary_options_layout.addLayout(self.summary_period_layout)
        self.summary_layout.addLayout(self.summary_options_layout)
        self.summary_tabs = QtWidgets.QTabWidget(summary)
        self.summary_tabs.setObjectName("summary_tabs")
        self.totals_tab = QtWidgets.QWidget()
        self.totals_tab.setObjectName("totals_tab")
        self.totals_tab_layout = QtWidgets.QFormLayout(self.totals_tab)
        self.totals_tab_layout.setObjectName("totals_tab_layout")
        self.distance_stat_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.distance_stat_label.setFont(font)
        self.distance_stat_label.setObjectName("distance_stat_label")
        self.totals_tab_layout.setWidget(
            0, QtWidgets.QFormLayout.LabelRole, self.distance_stat_label
        )
        self.total_distance_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.total_distance_label.setFont(font)
        self.total_distance_label.setObjectName("total_distance_label")
        self.totals_tab_layout.setWidget(
            0, QtWidgets.QFormLayout.FieldRole, self.total_distance_label
        )
        self.time_stats_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.time_stats_label.setFont(font)
        self.time_stats_label.setObjectName("time_stats_label")
        self.totals_tab_layout.setWidget(
            1, QtWidgets.QFormLayout.LabelRole, self.time_stats_label
        )
        self.total_time_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.total_time_label.setFont(font)
        self.total_time_label.setObjectName("total_time_label")
        self.totals_tab_layout.setWidget(
            1, QtWidgets.QFormLayout.FieldRole, self.total_time_label
        )
        self.activities_stat_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activities_stat_label.setFont(font)
        self.activities_stat_label.setObjectName("activities_stat_label")
        self.totals_tab_layout.setWidget(
            2, QtWidgets.QFormLayout.LabelRole, self.activities_stat_label
        )
        self.total_activities_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.total_activities_label.setFont(font)
        self.total_activities_label.setObjectName("total_activities_label")
        self.totals_tab_layout.setWidget(
            2, QtWidgets.QFormLayout.FieldRole, self.total_activities_label
        )
        self.climb_stats_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.climb_stats_label.setFont(font)
        self.climb_stats_label.setObjectName("climb_stats_label")
        self.totals_tab_layout.setWidget(
            3, QtWidgets.QFormLayout.LabelRole, self.climb_stats_label
        )
        self.total_climb_label = QtWidgets.QLabel(self.totals_tab)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.total_climb_label.setFont(font)
        self.total_climb_label.setObjectName("total_climb_label")
        self.totals_tab_layout.setWidget(
            3, QtWidgets.QFormLayout.FieldRole, self.total_climb_label
        )
        self.summary_tabs.addTab(self.totals_tab, "")
        self.records_tab = QtWidgets.QWidget()
        self.records_tab.setObjectName("records_tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.records_tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.records_table = RecordsTable(self.records_tab)
        self.records_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.records_table.setObjectName("records_table")
        self.records_table.setColumnCount(0)
        self.records_table.setRowCount(0)
        self.verticalLayout_2.addWidget(self.records_table)
        self.summary_tabs.addTab(self.records_tab, "")
        self.progression_tab = QtWidgets.QWidget()
        self.progression_tab.setObjectName("progression_tab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.progression_tab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.progression_chart = FullProgressionChart(self.progression_tab)
        self.progression_chart.setObjectName("progression_chart")
        self.verticalLayout_5.addWidget(self.progression_chart)
        self.summary_tabs.addTab(self.progression_tab, "")
        self.gallery_tab = QtWidgets.QWidget()
        self.gallery_tab.setObjectName("gallery_tab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.gallery_tab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gallery_scroll_area = QtWidgets.QScrollArea(self.gallery_tab)
        self.gallery_scroll_area.setWidgetResizable(True)
        self.gallery_scroll_area.setObjectName("gallery_scroll_area")
        self.gallery = Gallery()
        self.gallery.setGeometry(QtCore.QRect(0, 0, 96, 26))
        self.gallery.setObjectName("gallery")
        self.gallery_scroll_area.setWidget(self.gallery)
        self.verticalLayout_4.addWidget(self.gallery_scroll_area)
        self.summary_tabs.addTab(self.gallery_tab, "")
        self.heatmap_tab = QtWidgets.QWidget()
        self.heatmap_tab.setObjectName("heatmap_tab")
        self.heatmap_layout = QtWidgets.QVBoxLayout(self.heatmap_tab)
        self.heatmap_layout.setObjectName("heatmap_layout")
        self.summary_tabs.addTab(self.heatmap_tab, "")
        self.eddington_tab = QtWidgets.QWidget()
        self.eddington_tab.setObjectName("eddington_tab")
        self.eddington_tab_layout = QtWidgets.QVBoxLayout(self.eddington_tab)
        self.eddington_tab_layout.setObjectName("eddington_tab_layout")
        self.eddington_number_form = QtWidgets.QFormLayout()
        self.eddington_number_form.setObjectName("eddington_number_form")
        self.eddington_stats_label = QtWidgets.QLabel(self.eddington_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.eddington_stats_label.setFont(font)
        self.eddington_stats_label.setObjectName("eddington_stats_label")
        self.eddington_number_form.setWidget(
            0, QtWidgets.QFormLayout.LabelRole, self.eddington_stats_label
        )
        self.total_eddington_label = QtWidgets.QLabel(self.eddington_tab)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(False)
        font.setWeight(50)
        self.total_eddington_label.setFont(font)
        self.total_eddington_label.setObjectName("total_eddington_label")
        self.eddington_number_form.setWidget(
            0, QtWidgets.QFormLayout.FieldRole, self.total_eddington_label
        )
        self.eddington_tab_layout.addLayout(self.eddington_number_form)
        self.eddington_chart_widget = QtChart.QChartView(self.eddington_tab)
        self.eddington_chart_widget.setObjectName("eddington_chart_widget")
        self.eddington_tab_layout.addWidget(self.eddington_chart_widget)
        self.summary_tabs.addTab(self.eddington_tab, "")
        self.summary_layout.addWidget(self.summary_tabs)

        self.retranslateUi(summary)
        self.summary_period_box.setCurrentIndex(1)
        self.summary_tabs.setCurrentIndex(0)
        self.progression_chart.y_changed.connect(summary.summary_tab_switch)
        self.activity_types_list.itemChanged["QListWidgetItem*"].connect(
            summary.summary_tab_switch
        )
        self.activity_types_list.itemDoubleClicked["QListWidgetItem*"].connect(
            summary.summary_tab_switch
        )
        self.summary_tabs.currentChanged["int"].connect(summary.summary_tab_switch)
        self.summary_period_box.currentIndexChanged["QString"].connect(
            summary.summary_period_changed
        )
        QtCore.QMetaObject.connectSlotsByName(summary)

    def retranslateUi(self, summary):
        _translate = QtCore.QCoreApplication.translate
        self.label.setText(_translate("summary", "Period"))
        self.summary_period_box.setItemText(0, _translate("summary", "All Time"))
        self.summary_period_box.setItemText(1, _translate("summary", "Year"))
        self.summary_period_box.setItemText(2, _translate("summary", "Month"))
        self.summary_period_box.setItemText(3, _translate("summary", "Week"))
        self.distance_stat_label.setText(_translate("summary", "Distance"))
        self.total_distance_label.setText(_translate("summary", "TextLabel"))
        self.time_stats_label.setText(_translate("summary", "Time"))
        self.total_time_label.setText(_translate("summary", "TextLabel"))
        self.activities_stat_label.setText(_translate("summary", "Activities"))
        self.total_activities_label.setText(_translate("summary", "TextLabel"))
        self.climb_stats_label.setText(_translate("summary", "Climb"))
        self.total_climb_label.setText(_translate("summary", "TextLabel"))
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.totals_tab), _translate("summary", "Totals")
        )
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.records_tab),
            _translate("summary", "Records"),
        )
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.progression_tab),
            _translate("summary", "Progression"),
        )
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.gallery_tab),
            _translate("summary", "Gallery"),
        )
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.heatmap_tab),
            _translate("summary", "Heatmap"),
        )
        self.eddington_stats_label.setText(_translate("summary", "Eddington Number"))
        self.total_eddington_label.setText(_translate("summary", "TextLabel"))
        self.summary_tabs.setTabText(
            self.summary_tabs.indexOf(self.eddington_tab),
            _translate("summary", "Eddington"),
        )


from PyQt5 import QtChart
from activate.app.charts import FullProgressionChart
from activate.app.checklist import CheckList
from activate.app.photos import Gallery
from activate.app.tables import RecordsTable
