# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/ui/activity_summary.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_activity_summary(object):
    def setupUi(self, activity_summary):
        activity_summary.setObjectName("activity_summary")
        activity_summary.resize(634, 300)
        self.activity_summary_layout = QtWidgets.QGridLayout(activity_summary)
        self.activity_summary_layout.setObjectName("activity_summary_layout")
        self.date_activity_type_area = QtWidgets.QHBoxLayout()
        self.date_activity_type_area.setObjectName("date_activity_type_area")
        self.date_time_label = QtWidgets.QLabel(activity_summary)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.date_time_label.setFont(font)
        self.date_time_label.setObjectName("date_time_label")
        self.date_activity_type_area.addWidget(self.date_time_label)
        self.activity_type_label = QtWidgets.QLabel(activity_summary)
        self.activity_type_label.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.activity_type_label.setFont(font)
        self.activity_type_label.setObjectName("activity_type_label")
        self.date_activity_type_area.addWidget(self.activity_type_label)
        self.activity_summary_layout.addLayout(self.date_activity_type_area, 0, 1, 1, 1)
        self.info_table = InfoTable(activity_summary)
        self.info_table.setMinimumSize(QtCore.QSize(310, 0))
        self.info_table.setMaximumSize(QtCore.QSize(310, 16777215))
        self.info_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.info_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.info_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.info_table.setShowGrid(False)
        self.info_table.setColumnCount(3)
        self.info_table.setObjectName("info_table")
        self.info_table.setRowCount(0)
        self.info_table.horizontalHeader().setVisible(False)
        self.info_table.horizontalHeader().setDefaultSectionSize(100)
        self.info_table.horizontalHeader().setStretchLastSection(True)
        self.info_table.verticalHeader().setVisible(False)
        self.activity_summary_layout.addWidget(self.info_table, 2, 1, 1, 1)
        self.activity_text_area = QtWidgets.QVBoxLayout()
        self.activity_text_area.setObjectName("activity_text_area")
        self.activity_name_and_flags = QtWidgets.QHBoxLayout()
        self.activity_name_and_flags.setObjectName("activity_name_and_flags")
        self.activity_name_label = QtWidgets.QLabel(activity_summary)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activity_name_label.setFont(font)
        self.activity_name_label.setObjectName("activity_name_label")
        self.activity_name_and_flags.addWidget(self.activity_name_label)
        self.flags_label = QtWidgets.QLabel(activity_summary)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.flags_label.setFont(font)
        self.flags_label.setAlignment(
            QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
        )
        self.flags_label.setIndent(20)
        self.flags_label.setObjectName("flags_label")
        self.activity_name_and_flags.addWidget(self.flags_label)
        self.activity_text_area.addLayout(self.activity_name_and_flags)
        self.description_label = QtWidgets.QLabel(activity_summary)
        self.description_label.setWordWrap(True)
        self.description_label.setObjectName("description_label")
        self.activity_text_area.addWidget(self.description_label)
        self.activity_summary_layout.addLayout(self.activity_text_area, 0, 0, 1, 1)
        self.map_container = QtWidgets.QGridLayout()
        self.map_container.setObjectName("map_container")
        self.activity_summary_layout.addLayout(self.map_container, 1, 0, 2, 1)

        self.retranslateUi(activity_summary)
        QtCore.QMetaObject.connectSlotsByName(activity_summary)

    def retranslateUi(self, activity_summary):
        _translate = QtCore.QCoreApplication.translate
        activity_summary.setWindowTitle(_translate("activity_summary", "Form"))
        self.date_time_label.setText(_translate("activity_summary", "[Date]"))
        self.activity_type_label.setText(_translate("activity_summary", "[Type]"))
        self.activity_name_label.setText(
            _translate("activity_summary", "[Activity Name]")
        )
        self.flags_label.setText(_translate("activity_summary", "[Flags]"))
        self.description_label.setText(_translate("activity_summary", "[Description]"))


from activate.app.tables import InfoTable
