# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/main.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_main_window(object):
    def setupUi(self, main_window):
        main_window.setObjectName("main_window")
        main_window.resize(770, 507)
        self.main_widget = QtWidgets.QWidget(main_window)
        self.main_widget.setObjectName("main_widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.main_widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.main_tabs = QtWidgets.QTabWidget(self.main_widget)
        self.main_tabs.setObjectName("main_tabs")
        self.summary_tab = QtWidgets.QWidget()
        self.summary_tab.setObjectName("summary_tab")
        self.summary_tab_layout = QtWidgets.QHBoxLayout(self.summary_tab)
        self.summary_tab_layout.setObjectName("summary_tab_layout")
        self.summary = Summary(self.summary_tab)
        self.summary.setObjectName("summary")
        self.summary_tab_layout.addWidget(self.summary)
        self.main_tabs.addTab(self.summary_tab, "")
        self.activities_tab = QtWidgets.QWidget()
        self.activities_tab.setObjectName("activities_tab")
        self.activities_tab_layout = QtWidgets.QHBoxLayout(self.activities_tab)
        self.activities_tab_layout.setObjectName("activities_tab_layout")
        self.activity_list_area = QtWidgets.QVBoxLayout()
        self.activity_list_area.setObjectName("activity_list_area")
        self.activities_label = QtWidgets.QLabel(self.activities_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activities_label.setFont(font)
        self.activities_label.setObjectName("activities_label")
        self.activity_list_area.addWidget(self.activities_label)
        self.activity_list_table = ActivityListTable(self.activities_tab)
        self.activity_list_table.setMinimumSize(QtCore.QSize(500, 0))
        self.activity_list_table.setMaximumSize(QtCore.QSize(600, 16777215))
        self.activity_list_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.activity_list_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.activity_list_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.activity_list_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.activity_list_table.setCornerButtonEnabled(False)
        self.activity_list_table.setObjectName("activity_list_table")
        self.activity_list_table.setColumnCount(0)
        self.activity_list_table.setRowCount(0)
        self.activity_list_area.addWidget(self.activity_list_table)
        self.activities_tab_layout.addLayout(self.activity_list_area)
        self.activity_summary = ActivitySummary(self.activities_tab)
        self.activity_summary.setObjectName("activity_summary")
        self.activities_tab_layout.addWidget(self.activity_summary)
        self.main_tabs.addTab(self.activities_tab, "")
        self.social_tab = QtWidgets.QWidget()
        self.social_tab.setObjectName("social_tab")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.social_tab)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.social_tree = SocialTree(self.social_tab)
        self.social_tree.setObjectName("social_tree")
        self.social_tree.headerItem().setText(0, "1")
        self.verticalLayout.addWidget(self.social_tree)
        self.social_activity_list = SocialActivityList(self.social_tab)
        self.social_activity_list.setMinimumSize(QtCore.QSize(720, 0))
        self.social_activity_list.setMaximumSize(QtCore.QSize(600, 16777215))
        self.social_activity_list.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.social_activity_list.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.social_activity_list.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.social_activity_list.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.social_activity_list.setCornerButtonEnabled(False)
        self.social_activity_list.setObjectName("social_activity_list")
        self.social_activity_list.setColumnCount(0)
        self.social_activity_list.setRowCount(0)
        self.verticalLayout.addWidget(self.social_activity_list)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.social_activity_summary = ActivitySummary(self.social_tab)
        self.social_activity_summary.setObjectName("social_activity_summary")
        self.horizontalLayout_2.addWidget(self.social_activity_summary)
        self.main_tabs.addTab(self.social_tab, "")
        self.horizontalLayout.addWidget(self.main_tabs)
        main_window.setCentralWidget(self.main_widget)
        self.menubar = QtWidgets.QMenuBar(main_window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 770, 30))
        self.menubar.setObjectName("menubar")
        self.file_menu = QtWidgets.QMenu(self.menubar)
        self.file_menu.setObjectName("file_menu")
        self.export_menu = QtWidgets.QMenu(self.file_menu)
        self.export_menu.setObjectName("export_menu")
        self.settings_menu = QtWidgets.QMenu(self.menubar)
        self.settings_menu.setObjectName("settings_menu")
        self.activity_menu = QtWidgets.QMenu(self.menubar)
        self.activity_menu.setObjectName("activity_menu")
        main_window.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(main_window)
        self.statusbar.setObjectName("statusbar")
        main_window.setStatusBar(self.statusbar)
        self.action_import = QtWidgets.QAction(main_window)
        self.action_import.setObjectName("action_import")
        self.action_quit = QtWidgets.QAction(main_window)
        self.action_quit.setObjectName("action_quit")
        self.action_units = QtWidgets.QAction(main_window)
        self.action_units.setObjectName("action_units")
        self.action_edit = QtWidgets.QAction(main_window)
        self.action_edit.setObjectName("action_edit")
        self.export_original = QtWidgets.QAction(main_window)
        self.export_original.setObjectName("export_original")
        self.action_add_manual = QtWidgets.QAction(main_window)
        self.action_add_manual.setObjectName("action_add_manual")
        self.action_add_photos = QtWidgets.QAction(main_window)
        self.action_add_photos.setObjectName("action_add_photos")
        self.actionAdd = QtWidgets.QAction(main_window)
        self.actionAdd.setObjectName("actionAdd")
        self.action_servers = QtWidgets.QAction(main_window)
        self.action_servers.setObjectName("action_servers")
        self.action_analyse = QtWidgets.QAction(main_window)
        self.action_analyse.setObjectName("action_analyse")
        self.action_match = QtWidgets.QAction(main_window)
        self.action_match.setObjectName("action_match")
        self.action_as_route = QtWidgets.QAction(main_window)
        self.action_as_route.setObjectName("action_as_route")
        self.export_menu.addAction(self.export_original)
        self.export_menu.addAction(self.action_as_route)
        self.file_menu.addAction(self.action_import)
        self.file_menu.addAction(self.action_add_manual)
        self.file_menu.addAction(self.export_menu.menuAction())
        self.file_menu.addSeparator()
        self.file_menu.addAction(self.action_quit)
        self.settings_menu.addAction(self.action_units)
        self.settings_menu.addAction(self.action_servers)
        self.activity_menu.addAction(self.action_edit)
        self.activity_menu.addAction(self.action_add_photos)
        self.activity_menu.addAction(self.action_analyse)
        self.activity_menu.addAction(self.action_match)
        self.menubar.addAction(self.file_menu.menuAction())
        self.menubar.addAction(self.activity_menu.menuAction())
        self.menubar.addAction(self.settings_menu.menuAction())

        self.retranslateUi(main_window)
        self.main_tabs.setCurrentIndex(0)
        self.action_quit.triggered.connect(main_window.close)
        self.action_import.triggered.connect(main_window.import_activities)
        self.action_units.triggered.connect(main_window.edit_unit_settings)
        self.action_edit.triggered.connect(main_window.edit_activity_data)
        self.export_original.triggered.connect(main_window.export_activity)
        self.action_add_manual.triggered.connect(main_window.add_manual_activity)
        self.action_add_photos.triggered.connect(main_window.add_photos)
        self.action_servers.triggered.connect(main_window.edit_server_settings)
        self.action_analyse.triggered.connect(main_window.analyse_activity)
        self.activity_list_table.currentCellChanged['int','int','int','int'].connect(main_window.update_activity)
        self.main_tabs.currentChanged['int'].connect(main_window.main_tab_switch)
        self.social_tree.itemChanged['QTreeWidgetItem*','int'].connect(main_window.filter_social_activities)
        self.social_activity_list.currentCellChanged['int','int','int','int'].connect(main_window.update_social_activity)
        self.action_match.triggered.connect(main_window.match_activity)
        self.action_as_route.triggered.connect(main_window.export_as_route)
        QtCore.QMetaObject.connectSlotsByName(main_window)

    def retranslateUi(self, main_window):
        _translate = QtCore.QCoreApplication.translate
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.summary_tab), _translate("main_window", "Summary"))
        self.activities_label.setText(_translate("main_window", "Activities"))
        self.activity_list_table.setSortingEnabled(True)
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.activities_tab), _translate("main_window", "Activities"))
        self.social_activity_list.setSortingEnabled(True)
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.social_tab), _translate("main_window", "Social"))
        self.file_menu.setTitle(_translate("main_window", "File"))
        self.export_menu.setTitle(_translate("main_window", "Export"))
        self.settings_menu.setTitle(_translate("main_window", "Settings"))
        self.activity_menu.setTitle(_translate("main_window", "Activity"))
        self.action_import.setText(_translate("main_window", "Import"))
        self.action_import.setShortcut(_translate("main_window", "Ctrl+I"))
        self.action_quit.setText(_translate("main_window", "Quit"))
        self.action_units.setText(_translate("main_window", "Units"))
        self.action_edit.setText(_translate("main_window", "Edit"))
        self.export_original.setText(_translate("main_window", "Original"))
        self.action_add_manual.setText(_translate("main_window", "Add Manual"))
        self.action_add_manual.setShortcut(_translate("main_window", "Ctrl+M"))
        self.action_add_photos.setText(_translate("main_window", "Add Photos"))
        self.actionAdd.setText(_translate("main_window", "Add"))
        self.action_servers.setText(_translate("main_window", "Servers"))
        self.action_analyse.setText(_translate("main_window", "Analyse"))
        self.action_match.setText(_translate("main_window", "Find Matching"))
        self.action_as_route.setText(_translate("main_window", "As Route"))
from activate.app.activity_summary import ActivitySummary
from activate.app.social_tree import SocialTree
from activate.app.summary import Summary
from activate.app.tables import ActivityListTable, SocialActivityList
