# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/activity_view.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_activity_view(object):
    def setupUi(self, activity_view):
        activity_view.setObjectName("activity_view")
        activity_view.resize(567, 254)
        self.activity_view_layout = QtWidgets.QHBoxLayout(activity_view)
        self.activity_view_layout.setObjectName("activity_view_layout")
        self.map_container = QtWidgets.QGridLayout()
        self.map_container.setObjectName("map_container")
        self.activity_view_layout.addLayout(self.map_container)
        self.activity_tabs = QtWidgets.QTabWidget(activity_view)
        self.activity_tabs.setObjectName("activity_tabs")
        self.overview_tab = QtWidgets.QWidget()
        self.overview_tab.setObjectName("overview_tab")
        self.overview_tab_layout = QtWidgets.QHBoxLayout(self.overview_tab)
        self.overview_tab_layout.setObjectName("overview_tab_layout")
        self.activity_text_area = QtWidgets.QVBoxLayout()
        self.activity_text_area.setObjectName("activity_text_area")
        self.activity_name_label = QtWidgets.QLabel(self.overview_tab)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.activity_name_label.sizePolicy().hasHeightForWidth()
        )
        self.activity_name_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activity_name_label.setFont(font)
        self.activity_name_label.setObjectName("activity_name_label")
        self.activity_text_area.addWidget(self.activity_name_label)
        self.flags_label = QtWidgets.QLabel(self.overview_tab)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.flags_label.setFont(font)
        self.flags_label.setObjectName("flags_label")
        self.activity_text_area.addWidget(self.flags_label)
        self.date_time_label = QtWidgets.QLabel(self.overview_tab)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.date_time_label.setFont(font)
        self.date_time_label.setObjectName("date_time_label")
        self.activity_text_area.addWidget(self.date_time_label)
        self.activity_type_label = QtWidgets.QLabel(self.overview_tab)
        self.activity_type_label.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.activity_type_label.setFont(font)
        self.activity_type_label.setObjectName("activity_type_label")
        self.activity_text_area.addWidget(self.activity_type_label)
        self.description_label = QtWidgets.QLabel(self.overview_tab)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.description_label.sizePolicy().hasHeightForWidth()
        )
        self.description_label.setSizePolicy(sizePolicy)
        self.description_label.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop
        )
        self.description_label.setWordWrap(True)
        self.description_label.setObjectName("description_label")
        self.activity_text_area.addWidget(self.description_label)
        self.activity_text_area.setStretch(4, 1)
        self.overview_tab_layout.addLayout(self.activity_text_area)
        self.info_table = InfoTable(self.overview_tab)
        self.info_table.setMinimumSize(QtCore.QSize(310, 0))
        self.info_table.setMaximumSize(QtCore.QSize(310, 16777215))
        self.info_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.info_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.info_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.info_table.setShowGrid(False)
        self.info_table.setColumnCount(3)
        self.info_table.setObjectName("info_table")
        self.info_table.setRowCount(0)
        self.info_table.horizontalHeader().setVisible(False)
        self.info_table.horizontalHeader().setDefaultSectionSize(100)
        self.info_table.horizontalHeader().setStretchLastSection(True)
        self.info_table.verticalHeader().setVisible(False)
        self.overview_tab_layout.addWidget(self.info_table)
        self.activity_tabs.addTab(self.overview_tab, "")
        self.data_tab = QtWidgets.QWidget()
        self.data_tab.setMinimumSize(QtCore.QSize(100, 0))
        self.data_tab.setObjectName("data_tab")
        self.data_tab_layout = QtWidgets.QVBoxLayout(self.data_tab)
        self.data_tab_layout.setObjectName("data_tab_layout")
        self.graphs_layout = QtWidgets.QVBoxLayout()
        self.graphs_layout.setObjectName("graphs_layout")
        self.data_tab_layout.addLayout(self.graphs_layout)
        self.activity_tabs.addTab(self.data_tab, "")
        self.splits_tab = QtWidgets.QWidget()
        self.splits_tab.setObjectName("splits_tab")
        self.splits_tab_layout = QtWidgets.QVBoxLayout(self.splits_tab)
        self.splits_tab_layout.setObjectName("splits_tab_layout")
        self.split_table = SplitTable(self.splits_tab)
        self.split_table.setMinimumSize(QtCore.QSize(210, 0))
        self.split_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.split_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.split_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.split_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.split_table.setObjectName("split_table")
        self.split_table.setColumnCount(0)
        self.split_table.setRowCount(0)
        self.split_table.horizontalHeader().setSortIndicatorShown(True)
        self.split_table.verticalHeader().setVisible(False)
        self.splits_tab_layout.addWidget(self.split_table)
        self.activity_tabs.addTab(self.splits_tab, "")
        self.zones_tab = QtWidgets.QWidget()
        self.zones_tab.setObjectName("zones_tab")
        self.zones_tab_layout = QtWidgets.QVBoxLayout(self.zones_tab)
        self.zones_tab_layout.setObjectName("zones_tab_layout")
        self.zones_graph = QtChart.QChartView(self.zones_tab)
        self.zones_graph.setObjectName("zones_graph")
        self.zones_tab_layout.addWidget(self.zones_graph)
        self.activity_tabs.addTab(self.zones_tab, "")
        self.curve_tab = QtWidgets.QWidget()
        self.curve_tab.setObjectName("curve_tab")
        self.curve_tab_layout = QtWidgets.QHBoxLayout(self.curve_tab)
        self.curve_tab_layout.setObjectName("curve_tab_layout")
        self.curve_graph = QtChart.QChartView(self.curve_tab)
        self.curve_graph.setObjectName("curve_graph")
        self.curve_tab_layout.addWidget(self.curve_graph)
        self.curve_table = CurveTable(self.curve_tab)
        self.curve_table.setMaximumSize(QtCore.QSize(320, 16777215))
        self.curve_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.curve_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.curve_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.curve_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.curve_table.setObjectName("curve_table")
        self.curve_table.setColumnCount(0)
        self.curve_table.setRowCount(0)
        self.curve_table.horizontalHeader().setStretchLastSection(True)
        self.curve_table.verticalHeader().setVisible(False)
        self.curve_tab_layout.addWidget(self.curve_table)
        self.activity_tabs.addTab(self.curve_tab, "")
        self.activity_view_layout.addWidget(self.activity_tabs)

        self.retranslateUi(activity_view)
        self.activity_tabs.setCurrentIndex(0)
        self.activity_tabs.currentChanged["int"].connect(activity_view.update_page)
        QtCore.QMetaObject.connectSlotsByName(activity_view)

    def retranslateUi(self, activity_view):
        _translate = QtCore.QCoreApplication.translate
        activity_view.setWindowTitle(_translate("activity_view", "Analyse Activity"))
        self.activity_name_label.setText(_translate("activity_view", "[Activity Name]"))
        self.flags_label.setText(_translate("activity_view", "[Flags]"))
        self.date_time_label.setText(_translate("activity_view", "[Date]"))
        self.activity_type_label.setText(_translate("activity_view", "[Type]"))
        self.description_label.setText(_translate("activity_view", "[Description]"))
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.overview_tab),
            _translate("activity_view", "Overview"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.data_tab),
            _translate("activity_view", "Data"),
        )
        self.split_table.setSortingEnabled(True)
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.splits_tab),
            _translate("activity_view", "Splits"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.zones_tab),
            _translate("activity_view", "Zones"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.curve_tab),
            _translate("activity_view", "Curve"),
        )


from PyQt5 import QtChart

from activate.app.tables import CurveTable, InfoTable, SplitTable
